/* /////////////////////////////////////////////////////////////////////////////
//
//                  RC MODULE PROPRIETARY INFORMATION
//     This software is supplied under the terms of a license agreement or
//     nondisclosure agreement with RC MODULE and may not be copied
//     or disclosed except in accordance with the terms of that agreement.
//          Copyright (c) 2020 RC MODULE. All Rights Reserved.
//
////////////////////////////////////////////////////////////////////////////////
*/

#ifndef _NMDL_H_
#define _NMDL_H_

#define NMDL_MAX_INPUT_TENSORS 8
#define NMDL_MAX_OUTPUT_TENSORS 8
#define NMDL_MAX_UNITS 4

typedef enum tagNMDL_RESULT {
	NMDL_RESULT_OK,
	NMDL_RESULT_INVALID_FUNC_PARAMETER,
	NMDL_RESULT_NO_LOAD_LIBRARY,
	NMDL_RESULT_NO_BOARD,
	NMDL_RESULT_BOARD_RESET_ERROR,
	NMDL_RESULT_INIT_CODE_LOADING_ERROR,
	NMDL_RESULT_CORE_HANDLE_RETRIEVAL_ERROR,
	NMDL_RESULT_FILE_LOADING_ERROR,
	NMDL_RESULT_MEMORY_WRITE_ERROR,
	NMDL_RESULT_MEMORY_READ_ERROR,
	NMDL_RESULT_MEMORY_ALLOCATION_ERROR,
	NMDL_RESULT_MODEL_LOADING_ERROR,
	NMDL_RESULT_INVALID_MODEL,
	NMDL_RESULT_BOARD_SYNC_ERROR,
	NMDL_RESULT_BOARD_MEMORY_ALLOCATION_ERROR,
	NMDL_RESULT_NN_CREATION_ERROR,
	NMDL_RESULT_NN_LOADING_ERROR,
	NMDL_RESULT_NN_INFO_RETRIEVAL_ERROR,
	NMDL_RESULT_MODEL_IS_TOO_BIG,
	NMDL_RESULT_NOT_INITIALIZED,
	NMDL_RESULT_INCOMPLETE,
	NMDL_RESULT_UNKNOWN_ERROR
} NMDL_RESULT;

typedef enum tagNMDL_PROCESS_FRAME_STATUS {
	NMDL_PROCESS_FRAME_STATUS_FREE,
	NMDL_PROCESS_FRAME_STATUS_INCOMPLETE
} NMDL_PROCESS_FRAME_STATUS;

typedef enum tagNMDL_BOARD_TYPE {
	NMDL_BOARD_TYPE_SIMULATOR,
	NMDL_BOARD_TYPE_MC12101,
    NMDL_BOARD_TYPE_MC12705,
	NMDL_BOARD_TYPE_NMSTICK,
	NMDL_BOARD_TYPE_NMCARD,
	NMDL_BOARD_TYPE_NMMEZZO,
	NMDL_BOARD_TYPE_NMQUAD,
	NMDL_BOARD_TYPE_NMEMBED
} NMDL_BOARD_TYPE;

typedef struct tagNMDL_Tensor {
    unsigned int width;
    unsigned int height;
    unsigned int depth;
} NMDL_Tensor;

typedef struct tagNMDL_ModelInfo {
	unsigned int is_multi_unit;
    unsigned int input_tensor_num;
    NMDL_Tensor input_tensors[NMDL_MAX_INPUT_TENSORS];
    unsigned int output_tensor_num;
    NMDL_Tensor output_tensors[NMDL_MAX_OUTPUT_TENSORS];
} NMDL_ModelInfo;

typedef int NMDL_HANDLE;

#define NMDL_SIMULATOR_BOARD_NUMBER 0

#ifdef __cplusplus
extern "C" {
#endif

	NMDL_RESULT	NMDL_GetLibVersion(unsigned int *major, unsigned int *minor, unsigned int *patch);

	NMDL_RESULT NMDL_GetBoardCount(unsigned int board_type, unsigned int *boards);
	NMDL_RESULT NMDL_Blink(unsigned int board_type, unsigned int board_number);

	NMDL_RESULT NMDL_Create(NMDL_HANDLE *nmdl);
	void NMDL_Destroy(NMDL_HANDLE nmdl);

	NMDL_RESULT NMDL_Initialize(NMDL_HANDLE nmdl, unsigned int board_type, unsigned int board_number, unsigned int proc_number, const float *model[NMDL_MAX_UNITS], const unsigned int model_floats[NMDL_MAX_UNITS]);
	void NMDL_Release(NMDL_HANDLE nmdl);

	NMDL_RESULT	NMDL_GetModelInfo(NMDL_HANDLE nmdl, unsigned int unit_num, NMDL_ModelInfo *model_info);

	NMDL_RESULT	NMDL_Process(NMDL_HANDLE nmdl, unsigned int unit_num, const float *inputs[]);
	NMDL_RESULT NMDL_GetStatus(NMDL_HANDLE nmdl, unsigned int unit_num, unsigned int *status);
	NMDL_RESULT NMDL_GetOutput(NMDL_HANDLE nmdl, unsigned int unit_num, float *outputs[], double *fps);

	NMDL_RESULT	NMDL_ProcessStream(NMDL_HANDLE nmdl, unsigned int unit_num, const float *inputs[]);

#ifdef __cplusplus
}
#endif

#endif
